/**
 * Objects for fill-in-the-blank problems
 * for IQP JP 1102 at WPI
 * @author mqq
 */
package logic;

import java.io.*;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Pattern; 
import java.util.regex.Matcher;

public class FillProblem extends HwkProblem {
	
	/** Supports multiple answers */
	private List<AnswerStruct> answers = new LinkedList<AnswerStruct>();
	
	/** 
	 *  A counter for the number of blanks in this problem
	 *  Used for isValid() method 
	 */
	private int blankCounter = 0;
	
	/**
	 * Set up the problem before exporting the file
	 * For now, gives the name of the template;
	 * can be used for other things in the future
	 */
	protected void setUpProblem () {
		this.template = "fill_template_v3.txt";
	}
	
	/**
	 * Get the answers of the problem
	 * @return  The answers of the problem
	 */
	public List<AnswerStruct> getAnswers() {
		return answers;
	}
	
	/**
	 * Set the answers of the problem
	 * @param answers  The new answers
	 */
	public void setAnswer(List<AnswerStruct> answers) {
		this.answers = answers;
	}
	
	/**
	 * Add a new answer to the list of answers
	 * If gives a boolean, treat as a string
	 * true: case sensitive
	 * false: case insensitive
	 * @param answer  The new answer to be added
	 */
	public void addAnswer(String answer, boolean caseSensitive) {
		if (caseSensitive) {
			this.answers.add(new AnswerStruct("String", answer, 1));
		} else {
			this.answers.add(new AnswerStruct("String", answer, 0));
		}
	}
	
	/**
	 * Add a new answer to the list of answers
	 * If given another string for type after the answer
	 * Construct a AnswerStruct in that type,
	 * Give the tolerance of 0
	 * @param answer  The new answer to be added
	 * @param type  The type of the answer
	 */
	public void addAnswer(String answer, String type) {
		this.answers.add(new AnswerStruct(type, answer, 0));
	}
	
	/**
	 * Add a new answer to the list of answers
	 * If given a double after the answer
	 * Treat it as type numerical
	 * @param answer  The new answer to be added
	 * @param tolerance  The tolerance wanted
	 */
	public void addAnswer(String answer, double tolerance) {
		this.answers.add(new AnswerStruct("Numerical", answer, tolerance));
	}
	
	/**
	 * Remove a answer from the list of answers
	 * @param answer  The answer to be removed
	 */
	public void removeAnswer(AnswerStruct answer) {
		this.answers.remove(answer);
	}
	
	/**
	 * Gives the string to produce a answer box in PG
	 * The size of the blank is calculated based on the length of 
	 * the correct answer. 
	 * @param answer  The corresponding answer
	 * @return  The string to produce a answer box in PG
	 */
	public String blankString(AnswerStruct answer) {
		int answerLength = answer.getAnswer().length();
		int blankSize;
		// the size of the blank will always be a multiple of 10
		if (answerLength < 10) {
			blankSize = 10;
		} else {
			blankSize = (answerLength / 10 + 1) * 10;
		}
		return "\\\\{ ans_rule(" + blankSize + ") \\\\}";
	}
	
	/**
	 * Read from the template and embed the information
	 * Specialized for fill-in only
	 * @param input  The string from buffer
	 * @return  Modified string with information embedded
	 */
	protected String replaceTextSpecial(String input) {
		Pattern pAnswer = Pattern.compile("@AnsweR");
	    Matcher mAnswer = pAnswer.matcher(input);
	    if (mAnswer.find()) {
			String answerString = "";
			int i;
			for (i = 0; i < answers.size(); i++) {
				AnswerStruct answer_i = answers.get(i);
				String s_answer = ProblemFactory.substituteEscapingCharactersQuote(answer_i.getAnswer());
				// generate the string to check answers based on the type of a question
				if (answer_i.getType().equals("String")) {
					if (answer_i.getTolerance() == 0) {
						answerString = answerString + "      ANS(str_cmp(\"" + s_answer + "\" ));\n";
					} else {
						answerString = answerString + "      ANS(str_cmp(\"" + s_answer 
									+ "\", filters =>['compress_whitespace'] ));\n";
					}
				} else if (answer_i.getType().equals("Numerical")) {
					answerString = answerString + "      ANS(num_cmp(\"" 
						+ s_answer + "\",tol=>" + answer_i.getTolerance() + "));\n";
				} else if (answer_i.getType().equals("Expression")) {
					answerString = answerString + "      ANS(Compute(\"" + s_answer + "\")->cmp);\n";
				} else {
					System.out.println("ERROR: Unidentified Answer Type! Treated as String.");
					answerString = answerString + "      ANS(str_cmp(\"" + s_answer + "\" ));\n";
				}
			}
			input = mAnswer.replaceAll(answerString);
	    }
		Pattern pBlank = Pattern.compile("BLANK");
	    Matcher mBlank = pBlank.matcher(input);
	    while (mBlank.find()) {
			input = mBlank.replaceFirst(this.blankString(answers.get(blankCounter)));
			blankCounter++;
			mBlank = pBlank.matcher(input);
	    }
		return input;
	}
	
	/**
	 * Check if the problem is valid
	 * For fill-in, check if the number of answers matches the number of blanks
	 */
	protected boolean isValid() {
		return this.answers.size() == this.blankCounter && this.answers.size() > 0;
	}
	
	/**
	 * Import from a string
	 * For fill-in, set up the list of answers
	 */
	protected void importFromLineSpecial(String line) {
		if (line.contains("ANS(str_cmp(\"")) {
			if (!line.contains("filters")) {
				String stringAnswer = ProblemFactory.extractBetween(line, "ANS(str_cmp(\"", "\" ));");
				this.addAnswer(ProblemFactory.substituteEscapingCharactersQuoteBack(stringAnswer), false);
			} else {
				String stringAnswer = ProblemFactory.extractBetween(line, "ANS(str_cmp(\"", "\", filters =>['compress_whitespace'] ));");
				this.addAnswer(ProblemFactory.substituteEscapingCharactersQuoteBack(stringAnswer), true);
			}
		} else if (line.contains("ANS(num_cmp")) {
			String numberAnswer = ProblemFactory.extractBetween(line, "ANS(num_cmp(\"", "\",tol=>");
			numberAnswer = ProblemFactory.substituteEscapingCharactersQuoteBack(numberAnswer);
			double tolerance = Double.parseDouble(ProblemFactory.extractBetween(line, "\",tol=>", "));"));
			this.addAnswer(numberAnswer, tolerance);
		} else if (line.contains("ANS(Compute(\"")) {
			String expAnswer = ProblemFactory.extractBetween(line, "ANS(Compute(\"", "\")->cmp);");
			expAnswer = ProblemFactory.substituteEscapingCharactersQuoteBack(expAnswer);
			this.addAnswer(expAnswer, "Expression");
		}
	}

	/**
	 * Fix problems that could occur during importing
	 */
	protected void adjust() {
		// remove the extra "\n" at the end
		this.body = this.body.substring(0, this.body.length() - 1);
		// for fill-in only
		this.body = this.body.replace("             $PAR ", "");
		// for fill-in only
		// substitute the answer box syntax back to "BLANK"
		while (this.body.contains("ans_rule")) {			
			String blankSize = ProblemFactory.extractBetween(this.body, "\\{ ans_rule(", ") \\}");
			System.out.println(blankSize);
			this.body = this.body.replace("\\{ ans_rule(" + blankSize + ") \\}", "BLANK");
		}
		if (this.solution != null) {
			// remove the three extra "\n" at the beginning
			this.solution = this.solution.substring(3);
			// remove the extra "\n" at the end
			this.solution = this.solution.substring(0, this.solution.length() - 1);
		}
	}
	
	/*public static void main(String[] args) throws IOException {
		FillProblem fp = new FillProblem();
		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
		System.out.println("Please enter the header:");
		fp.setHeader(in.readLine());
		System.out.println("Please enter the problem:");
		fp.setBody(in.readLine());
		System.out.println("Please enter the answer:");
		fp.addAnswer(in.readLine(), true);
		fp.addAnswer(in.readLine(), "Expression");
		fp.addAnswer(in.readLine(), 1);
		System.out.println("Please enter the sol:");
		fp.setSolution(in.readLine());
		System.out.println("Please enter the file name:");
		fp.exportFile(in.readLine());
	}*/
	/** tests */
	public static void main(String[] args) {
		MultiProblem m = new MultiProblem();
		m.setBody("bodybody");
		m.setHeader("hdhd");
		m.addAnswer("aaa");
		m.addAnswer("ccc");
		m.addChoice("bbb");
		m.addChoice("ddd");
		m.setSolution("sol\nsol2\nsol3");
		m.setShowPartialCorrectAnswers(1);
		m.setIsCheckBox(true);
		m.exportFile("C:\\Users\\QQ\\Desktop\\webworktest\\MCtest.pg");
		MultiProblem mp = (MultiProblem) HwkProblem.importFile("C:\\Users\\QQ\\Desktop\\webworktest\\MCtest.pg");
		mp.exportFile("C:\\Users\\QQ\\Desktop\\webworktest\\MCtest2.pg");
		FillProblem f = new FillProblem();
		f.setHeader("header");
		f.setBody("1 + 1 = BLANK\na + b = BLANK\nx + x = BLANK");
		f.addAnswer("2", 0);
		f.addAnswer("C", true);
		f.addAnswer("2x", "Expression");
		f.exportFile("C:\\Users\\QQ\\Desktop\\webworktest\\FLtest.pg");
		FillProblem fp = (FillProblem) HwkProblem.importFile("C:\\Users\\QQ\\Desktop\\webworktest\\FLtest.pg");
		System.out.println("START   " + fp.header);
		System.out.println(fp.body);
		System.out.println(fp.answers.get(0).toString());
		System.out.println(fp.answers.get(1).toString());
		System.out.println(fp.answers.get(2).toString());
		fp.exportFile("C:\\Users\\QQ\\Desktop\\webworktest\\FLtest2.pg");
		MatchingProblem ma = new MatchingProblem();
		ma.addQa("AAA", "aaa");
		ma.addQa("BBB", "bbb");
		ma.addQa("CCC", "ccc");
		ma.setBody("\\(x^2+1\\)\nlalalalu\n");
		ma.setHeader("$$mama\"mum\"");
		ma.setShowPartialCorrectAnswers(1);
		ma.exportFile("C:\\Users\\QQ\\Desktop\\webworktest\\MAtest.pg");
		MatchingProblem map = (MatchingProblem) HwkProblem.importFile("C:\\Users\\QQ\\Desktop\\webworktest\\MAtest.pg");
		map.exportFile("C:\\Users\\QQ\\Desktop\\webworktest\\MAtest2.pg");

	}
}

