package logic;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.InputStreamReader;

/**
 * Basic object for WeBWorK problems to be generated as .pg file
 * for IQP JP 1102 at WPI
 * @author mqq
 */

public abstract class HwkProblem {
	/** Problem header */
	protected String header;
	/** Problem statement */
	protected String body;
	/** 
	 * Used when importing
	 * 0:	Default
	 * 1:	Reading the problem body
	 * 2:	Reading the problem solution
	 * 3:	Reading matching problem qas
	 */
	protected int importingMode = 0;
	/**
	 * 0:	Yes
	 * 1:	No
	 */
	protected int showPartialCorrectAnswers = 0;
	/**
	 * Different templates are used for different problems
	 * Set by setUpProblem()
	 */
	protected String template;
	/**
	 * Problem solution
	 * leave it null if don't want solution
	 */
	protected String solution = null; 
	
	/**
	 * Set the header of the problem
	 * @param header  The header of the problem
	 */
	public void setHeader(String header) {
		this.header = header;
	}
	
	/**
	 * Set the body of the problem
	 * @param body  the body of the problem
	 */
	public void setBody(String body) {
		this.body = body;
	}
        
        /**
	 * Get the body of the problem
	 */
	public String getBody() {
		return body;
	}
	
	/**
	 * Set the boolean indicating whether to show partial correct answers
	 * @param showPartialCorrectAnswers  1 if yes; 0 otherwise
	 */
	public void setShowPartialCorrectAnswers(int showPartialCorrectAnswers) {
		this.showPartialCorrectAnswers = showPartialCorrectAnswers;
	}
        
        public int getPCA(){
            return showPartialCorrectAnswers;
        }
	
	/**
	 * Set the solution of the problem
	 * @param body  the solution of the problem
	 */
	public void setSolution(String solution) {
		this.solution = solution;
	}
        
        public String getSolution(){
            return solution;
        }
	
	/**
	 * Read from the template and embed the information
	 * Common for all problems
	 * @param input  The string from buffer
	 * @return  Modified string with information embedded
	 */
	protected String replaceTextCommon(String input) {
		String s_header = ProblemFactory.substituteEscapingCharactersQuote(header);
		input = input.replace("@HeadeR", s_header);
		String s_body = ProblemFactory.substituteEscapingCharactersText(body);
		input = input.replace("@BodY", s_body);
		input = input.replace("@PartiaL", showPartialCorrectAnswers+"");
		if (solution != null) {
			String s_solution = ProblemFactory.substituteEscapingCharactersText(solution);
			input = input.replace("@SolutioN", "SOLUTION(EV3(<<'END_SOLUTION'));\n$PAR\n$SOL\n$PAR\n" + 
					s_solution + "\nEND_SOLUTION\n");
		} else {
			input = input.replace("@SolutioN", "");
		}
		return input;
	}
	
	/**
	 * Read from the template and embed the information
	 * Contains features that are unique for different types of problem
	 * @param input  The string from buffer
	 * @return  Modified string with information embedded
	 */
	abstract protected String replaceTextSpecial(String input);
	
	/**
	 * Check if the problem is valid
	 * @return  true if yes; false if no
	 */
	abstract protected boolean isValid();
	
	/**
	 * Set up the template path for now
	 */
	abstract protected void setUpProblem();
	
	/**
	 * Export the problem into a pg file
	 * @param fileName  The file name to be used
	 * @return 	 0 if successful
	 * 			 1 if the file name has been used by another file
	 * 			 2 if the homework problem is not valid
	 */
	public int exportFile(String fileName) {
		try {
			this.setUpProblem();
			InputStream fstream = logic.HwkProblem.class.getResourceAsStream(template);
			// Get the object of DataInputStream
			DataInputStream in = new DataInputStream(fstream);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			String strLine;
			// Create new file with the given name
			File file = new File(fileName);
			boolean exist = file.createNewFile();
			if (!exist) {
				System.out.println("ERROR: File already exists!");
				return 1;
			}
			else {
				FileWriter fwriter = new FileWriter(fileName);
				BufferedWriter out = new BufferedWriter(fwriter);
				System.out.println("File created successfully.");
				// Read File Line By Line
				while ((strLine = br.readLine()) != null)   {
					strLine = this.replaceTextCommon(strLine);
					out.write(this.replaceTextSpecial(strLine));
					out.newLine();
				}
				//Close the input and out put stream
				out.close();
				in.close();
			}
		} catch (Exception e){ //Catch exception if any
			System.err.println("Error: " + e.getMessage());
		}
		if (this.isValid()) {
			return 0;
		} else {
			System.out.println("WARNING: Inappropriate Homework Problem!");
			return 2;
		}
	}
	
	/**
	 * Import from a file
	 * Line by line using buffered reader
	 * @param fileName  The file to be imported
	 * @return  A certain type of homework problem
	 */
	public static HwkProblem importFile(String fileName) {
		try {
			FileInputStream fstream = new FileInputStream(fileName);
			// Get the object of DataInputStream
			DataInputStream in = new DataInputStream(fstream);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			String strLine;
			HwkProblem newProblem;
			// Read File Line By Line
			if ((strLine = br.readLine()) == null) {
				System.out.println("Import failed! Error reading from file!");
				return null;
			}
			// Need to make sure file was generated by this program
			// The difference of a single space could lead to error 
			if (!strLine.contains("# This file is generated by WbWrkGUI")) {
				System.out.println("Import failed! This file is not generated by this program.");
				return null;
			} else {
				strLine = br.readLine();
				// first determine type
				if (strLine.contains("# This is a fill-in problem")) {
					newProblem = new FillProblem();
					while ((strLine = br.readLine()) != null) {
						newProblem.importFromLineSpecial(strLine);
						newProblem.importFromLine(strLine);
					}
				} else if (strLine.contains("# This is a matching problem")) {
					newProblem = new MatchingProblem();
					while ((strLine = br.readLine()) != null) {
						newProblem.importFromLineSpecial(strLine);
						newProblem.importFromLine(strLine);
					}
				} else if (strLine.contains("# This is a multiple choice problem")) {
					newProblem = new MultiProblem();
					while ((strLine = br.readLine()) != null) {
						newProblem.importFromLineSpecial(strLine);
						newProblem.importFromLine(strLine);
					}
				} else {
					System.out.println("Import failed! Unrecognized problem type!");
					return null;
				}
			}
			//Close the input and out put stream
			in.close();
			// fix errors
			newProblem.adjust();
			return newProblem;
		} catch (Exception e){ //Catch exception if any
			System.err.println("Error: " + e.getMessage());
			return null;
		}
	}
	
	/**
	 * Fix problems that could occur during importing
	 */
	abstract protected void adjust();
	
	/**
	 * Import from a line
	 * Contains features that are unique for different types of problem
	 * @param line  The line input
	 */
	abstract protected void importFromLineSpecial(String line);
	
	/**
	 * Import from a line
	 * @param line  The line input
	 */
	private void importFromLine(String line) {
		if (importingMode == 1) {
			if (line.contains("END_TEXT")) {
				this.importingMode = 0;
			} else {
				this.body = this.body + ProblemFactory.substituteEscapingCharactersTextBack(line) + "\n";
			}
		} else if (importingMode == 2) {
			if (line.contains("END_SOLUTION")) {
				this.importingMode = 0;
			} else {
				this.solution = this.solution + ProblemFactory.substituteEscapingCharactersTextBack(line.replace("$PAR", "").replace("$SOL", "")) + "\n";		
			}
		}
		if (line.contains("$showPartialCorrectAnswers")) {
			// the 29th char is the number
			this.showPartialCorrectAnswers = Integer.parseInt((line.charAt(29))+"");
		} else if (line.contains("TEXT(beginproblem()")) {
			this.header = ProblemFactory.extractBetween(line, 
					"TEXT(beginproblem(), $BR, $BBOLD, \"", 
					"\", $EBOLD, $BR, $BR);");
			this.header = ProblemFactory.substituteEscapingCharactersQuoteBack(this.header);
		} else if (line.contains("BEGIN_TEXT")) {
			this.body = "";
			this.importingMode = 1;
		} else if (line.contains("SOLUTION(EV3(<<'END_SOLUTION'));")) {
			this.solution = "";
			this.importingMode = 2;
		} 
	}
}
