/**
 * Objects for multiple choice problems
 * for IQP JP 1102 at WPI
 * @author mqq
 */
package logic;
import java.util.LinkedList;
import java.util.List;

public class MultiProblem extends HwkProblem {
	/** List of answers */
	private List<String> answers = new LinkedList<String>();
	/** List of other choices */
	private List<String> choices = new LinkedList<String>();
	
	/** 
	 * A choice that will always stays at the bottom 
	 * Leave as null if don't want one
	 */
	private String bottomChoice;
	
	/**
	 * true:	If allow the students to enter multiple answers
	 * false:	If not
	 */
	private boolean isCheckBox = false;
	
	/** Used when importing, find the third line after "$cmc->qa" */
	private int importingLineCounter = 0;

	/**
	 * Set up the problem before exporting the file
	 * For now, gives the name of the template;
	 * can be used for other things in the future
	 */
	protected void setUpProblem () {
		this.template = "multi_template_v3.txt";
	}
	
	/**
	 * Set the answers of the problem
	 * @param answers  The new answers
	 */
	public void setAnswers(List<String> answers) {
		this.answers = answers;
	}
        
        public List<String> getAnswers(){
            return answers;
        }
	
	/**
	 * Add a new answer to the list
	 * @param newChoice  The new answer to be added
	 */
	public void addAnswer(String newAnswer) {
		this.answers.add(newAnswer);
	}

	/**
	 * Set the choices of the problem
	 * @param choices  The new choices
	 */
	public void setChoices(List<String> choices) {
		this.choices = choices;
	}
	
        public List<String> getChoices(){
            return choices;
        }
	/**
	 * Add a new choice to the list
	 * @param newChoice  The new choice to be added
	 */
	public void addChoice(String newChoice) {
		this.choices.add(newChoice);
	}
	
	/**
	 * Remove a choice from the list
	 * @param choice  The choice to be removed
	 */
	public void removeChoice(String choice) {
		this.choices.remove(choice);
	}
	
	/**
	 * Remove a answer from the list
	 * @param answer  The answer to be removed
	 */
	public void removeAnswer(String answer) {
		this.answers.remove(answer);
	}
	
	/**
	 * Mark a choice from either list as a bottom choice
	 * @param choice  The expected bottom choice
	 */
	public void markBottomChoice(String choice) {
		this.bottomChoice = choice;
	}
        
        public String getBottomChoice(){
            return bottomChoice;
        }
	
	/**
	 * Set the isCheckBox flag to a new boolean
	 * @param isCheckBox  The new boolean
	 */
	public void setIsCheckBox(boolean isCheckBox) {
		this.isCheckBox = isCheckBox;
	}
        
        public boolean getIsCheckBox(){
            return isCheckBox;
        }
	
	/**
	 * Check if the problem is valid
	 * For multiple choice, check whether it is check box if there are more than
	 * one correct answer
	 */
	protected boolean isValid() {
		if (this.answers.size() < 1) {
			return false;
		} else if (this.answers.size() == 1) {
			return true;
		} else {
			return isCheckBox;
		}
	}
	
	/**
	 * Read from the template and embed the information
	 * @param input  The string from buffer
	 * @return  Modified string with information embedded
	 */
	protected String replaceTextSpecial(String input) {
		String s_question = ProblemFactory.substituteEscapingCharactersQuote(body);
		input = input.replace("@QuestioN", s_question);
		String answerString = ProblemFactory.listToString(answers);
		input = input.replace("@AnsweR", answerString);
		String choicesString = ProblemFactory.listToString(choices);
		input = input.replace("@ChoiceS", choicesString);
		if (isCheckBox) {
			input = input.replace("new_multiple_choice", "new_checkbox_multiple_choice");
			input = input.replace("radio_cmp", "checkbox_cmp");
		}
		if (bottomChoice != null) {
			String s_bottomChoice = ProblemFactory.substituteEscapingCharactersQuote(bottomChoice);
			input = input.replace("@BottoM", s_bottomChoice);
		} else {
			input = input.replace("$cmc->makeLast(\"@BottoM\");", "");
		}
		return input;
	}
	
	/**
	 * Set up the choices for multiple choice
	 */
	protected void importFromLineSpecial(String line) {
		if (this.importingLineCounter != 0) {
			this.importingLineCounter++;
		}
		if (this.importingLineCounter == 3) {
			this.answers = ProblemFactory.stringToList(line);
		}
		if (line.contains("new_checkbox_multiple_choice")) {
			this.isCheckBox = true;
		} else if (line.contains("$cmc->qa(")) {
			this.importingLineCounter = 1;
		} else if (line.contains("$cmc->extra(")) {
			String choicesString = ProblemFactory.extractBetween(line, "$cmc->extra(", ");");
			this.choices = ProblemFactory.stringToList(choicesString);
		} else if (line.contains("$cmc->makeLast(")) {
			String bottomString = ProblemFactory.extractBetween(line, "$cmc->makeLast(\"", "\");");
			this.bottomChoice = ProblemFactory.substituteEscapingCharactersQuoteBack(bottomString);
		}
	}
	
	/**
	 * Fix problems that could occur during importing
	 */
	protected void adjust() {
		this.body = this.body.replace("$PAR\n\\{$cmc->print_a\\}\n", "");
		this.body = this.body.substring(0, this.body.length() - 1);
		if (this.solution != null) {
			this.solution = this.solution.substring(3);
			this.solution = this.solution.substring(0, this.solution.length() - 1);
		}
	}
	/*public static void main(String[] args) throws IOException {
		MultiProblem mp = new MultiProblem();
		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
		System.out.println("Please enter the header:");
		mp.setHeader(in.readLine());
		System.out.println("Please enter the problem:");
		mp.setBody(in.readLine());
		System.out.println("Please enter the answer (this test only supports 2):");
		mp.addAnswer(in.readLine());
		mp.addAnswer(in.readLine());
		System.out.println("Please enter the wrong answers (this test only supports 3):");
		mp.addChoice(in.readLine());
		mp.addChoice(in.readLine());
		mp.addChoice(in.readLine());
		System.out.println("Please enter the bottom answer:");
		mp.markBottomChoice(in.readLine());
		System.out.println("This is a check box.");
		mp.setIsCheckBox(true);
		System.out.println("Please enter the file name:");
		mp.exportFile(in.readLine());
	}*/

}
